import sys
from PyQt5 import QtWidgets,  QtCore
from PyQt5.QtWidgets import QMainWindow,  QFileDialog
from PyQt5.QtGui import QIcon, QPixmap


from Ui_editor_ui import Ui_MainWindow
class MainWindow(QMainWindow, Ui_MainWindow):
    def __init__(self, win):
        QMainWindow.__init__(self)
        self.setupUi(win)
        self.active_type = '2'
        self.bg = QPixmap('data/images/bg.png')
        self.scale = .5
        self.pushButtons = []
        self.blocks = {}
        self.kills = {}
        self.fills ={}
        self.others = {}
        self.entities = {}
        self.load_legend()
        self.comboBox.addItem("Blocks")
        self.comboBox.addItem("Kills")
        self.comboBox.addItem("Fills")
        self.comboBox.addItem("Items")
        self.comboBox.addItem("Entities")
        self.comboBox.activated[str].connect(self.select_type)
        self.comboBox.setCurrentIndex(0)
        self.select_type('Blocks')
        self.map = self.load_map('data/map.txt')
        self.scene = GraphicsScene(self)
        self.actionNew.triggered.connect(self.new)
        self.actionOpen.triggered.connect(self.open)
        self.actionSave.triggered.connect(self.save)
        self.actionExit.triggered.connect(self.exit)
        self.actionFull.triggered.connect( lambda: self.set_scale(1) )  
        self.action3_4.triggered.connect(lambda: self.set_scale(.75) )  
        self.action1_2.triggered.connect(lambda: self.set_scale(.5) )  
        self.action1_4.triggered.connect(lambda: self.set_scale(.25) )     
        self.actionOpen_Image.triggered.connect(self.open_image)
   
        self.draw_map()
        self.set_scene()
        
    def new(self):
        pass

    def open(self):
        fname = QFileDialog.getOpenFileName(self, 'Open file', './data/')
        self.map = self.load_map(fname[0])
        self.draw_map()
        self.set_scene()
        
        
    def save(self, text):
        fname = QFileDialog.getSaveFileName(self, 'Save file', './data/')
        self.map = self.save_map(fname[0])

    def exit(self, text):
        sys.exit(app.exec_())
        
    def set_scale(self, scale):
        self.scale = scale
        self.draw_map()
        self.set_scene()
        
    def open_image(self):
        fname = QFileDialog.getOpenFileName(self, 'Open file', './data/')
        self.bg = QPixmap(fname[0])
        self.draw_map()
        self.set_scene()


    def select_type(self, type):
        if type == "Blocks":
            self.load_block_buttons(self.blocks)
        if type == "Kills":
            self.load_block_buttons(self.kills)
        if type == "Fills":
            self.load_block_buttons(self.fills)
        if type == "Items":
            self.load_block_buttons(self.others)
        if type == "Entities":
            self.load_block_buttons(self.entities)

    def load_block_buttons(self, blocks):
        for butt in self.pushButtons:
            butt.deleteLater()
        self.pushButtons = []
        i = 0
        for block in blocks:
            self.pushButtons.append(QtWidgets.QRadioButton(self.scrollAreaWidgetContents))
            self.pushButtons[i].setMinimumSize(QtCore.QSize(90, 64))
            self.pushButtons[i].setMaximumSize(QtCore.QSize(90, 64))
            self.pushButtons[i].setIconSize(QtCore.QSize(56, 56))
            self.pushButtons[i].flag = block
            self.pushButtons[i].setIcon(QIcon(QPixmap('data/images/' + blocks[block])))
            self.pushButtons[i].toggled.connect(self.toggled)
            self.verticalLayout.addWidget(self.pushButtons[i])
            i += 1
    def toggled(self):
        radioButton = self.sender()
        if radioButton.isChecked():
            self.active_type = radioButton.flag

    def load_legend(self):
        f = open('data/legend.txt','r')
        data = f.read()
        f.close()
        data = data.split('\n')
        for r in data:
            c = r.split(';')
            if len(c) >= 3:
                if c[2] == 's':
                    self.blocks[c[0]] = c[1]
                elif c[2] == 'k':
                    self.kills[c[0]] = c[1]
                elif c[2] == 'o':
                    self.fills[c[0]] = c[1]
                elif c[2] == 'e':
                    self.entities[c[0]] = c[1]
                else:
                    self.others[c[0]] = c[1]

    def load_map(self, path):
        f = open(path,'r')
        data = f.read()
        f.close()
        data = data.split('\n')
        game_map = []
        for row in data:
            game_map.append(list(row))
        return game_map

    def save_map(self, path):
        rows = []
        for r in self.map:
            rows.append("".join(r))
        txt = '\n'.join(rows)
        f = open(path,'w')
        f.write(txt)
        f.close()


    def draw_map(self):
        bgSize = (int(len(self.map[0]) * 64 * self.scale), int(len(self.map) * 64 * self.scale))
        self.scene = GraphicsScene(self)
        self.scene.addItem(QtWidgets.QGraphicsPixmapItem(self.bg.scaled(bgSize[0], bgSize[1])))
        image = [[0]*len(self.map)]*len(self.map[0])
        scaled_size = int(64*self.scale)
        y = 0
        for layer in self.map:
            x = 0
            for tile in layer:
                for b in self.blocks:
                    if tile == b:
                        image[x][y] = QtWidgets.QGraphicsPixmapItem(QPixmap('data/images/' + self.blocks[b]).scaled(scaled_size,scaled_size ))
                        image[x][y].setPos(x*64 * self.scale,y*64 * self.scale)
                        self.scene.addItem(image[x][y])
                for k in self.kills:
                    if tile == k:
                        image[x][y] = QtWidgets.QGraphicsPixmapItem(QPixmap('data/images/' + self.kills[k]).scaled(scaled_size,scaled_size ))
                        image[x][y].setPos(x*64 * self.scale,y*64 * self.scale)
                        self.scene.addItem(image[x][y])
                for f in self.fills:
                    if tile == f:
                        image[x][y] = QtWidgets.QGraphicsPixmapItem(QPixmap('data/images/' + self.fills[f]).scaled(scaled_size,scaled_size ))
                        image[x][y].setPos(x*64 * self.scale,y*64 * self.scale)
                        self.scene.addItem(image[x][y])
                for o in self.others:
                    if tile == o:
                        image[x][y] = QtWidgets.QGraphicsPixmapItem(QPixmap('data/images/' + self.others[o]).scaled(scaled_size,scaled_size ))
                        image[x][y].setPos(x*64 * self.scale,y*64 * self.scale)
                        self.scene.addItem(image[x][y])
                for e in self.entities:
                    if tile == e:
                        image[x][y] = QtWidgets.QGraphicsPixmapItem(QPixmap('data/images/' + self.entities[e]).scaled(scaled_size,scaled_size ))
                        image[x][y].setPos(x*64 * self.scale,y*64 * self.scale)
                        self.scene.addItem(image[x][y])
                x += 1
            y += 1
        
    def set_scene(self):
        self.graphicsView.setScene(self.scene)
        
    def change_tile(self,x,y,b):
        if b == 1:
            self.map[int(y//(64 * self.scale))][int(x//(64 * self.scale))] = self.active_type
        if b == 2:
            self.map[int(y//(64 * self.scale))][int(x//(64 * self.scale))] = '0'
        if b == 4:
            self.active_type = self.map[int(y//(64 * self.scale))][int(x//(64 * self.scale))]
        self.scene.clear()
        self.draw_map()
        self.set_scene()

class GraphicsScene(QtWidgets.QGraphicsScene):
    def __init__(self, parent=None):
        QtWidgets.QGraphicsScene.__init__(self, parent)
        
    def mousePressEvent(self, event):
        x = event.scenePos().x()
        y = event.scenePos().y()
        prog.change_tile(x, y, event.button())
 #       print(f"x {x} y {y} {event.button()}")



if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    win = QMainWindow()
    prog = MainWindow(win)
    win.show()
    sys.exit(app.exec_())
